import logging
from studiopulsee.app import config

if config.LOGS_MODE == "ovh":
    try:
        from pygelf import GelfTcpHandler, gelf
    except ModuleNotFoundError:
        print("pygelf module not found.")


def configure_logs_ovh(app):
    class GelfOVHHandler(GelfTcpHandler):
        """
        Class to adapt OVH Log management platform
        """

        def __init__(self, ovh_token, **kwargs):
            """
            OVH TCP Handler
            :param ovh_token: OVH API Token
            """
            self.ovh_token = ovh_token
            GelfTcpHandler.__init__(self, **kwargs)

        def convert_record_to_gelf(self, record):
            l = gelf.make(
                record,
                self.domain,
                self.debug,
                self.version,
                self.additional_fields,
                self.include_extra_fields,
            )
            l.update({"_X-OVH-TOKEN": self.ovh_token})
            return gelf.pack(l, self.compress, self.json_default)

    graylogHandler = GelfOVHHandler(
        host=app.config["LOGS_HOST"],
        port=app.config["LOGS_PORT"],
        ovh_token=app.config["LOGS_TOKEN"],
    )
    graylogHandler.setLevel(logging.ERROR)
    app.logger.addHandler(graylogHandler)



from studiopulsee.app.models.preview_file import PreviewFile
from studiopulsee.app.models.task import Task
from studiopulsee.app.models.project import Project
from studiopulsee.app.services import (
    user_service,
    tasks_service,
)
from studiopulsee.app.utils import permissions

from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource


class PreviewFilesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, PreviewFile)

    def add_project_permission_filter(self, query):
        if permissions.has_vendor_permissions():
            query = (
                query.join(Task)
                .filter(user_service.build_assignee_filter())
                .filter(user_service.build_open_project_filter())
            )
        elif not permissions.has_admin_permissions():
            query = (
                query.join(Task)
                .join(Project)
                .filter(user_service.build_related_projects_filter())
                .filter(user_service.build_open_project_filter())
            )

        return query

    def check_read_permissions(self):
        return True


class PreviewFileResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, PreviewFile)

    def check_read_permissions(self, preview_file):
        """
        If it's a vendor, check if the user is working on the task.
        If it's an artist, check if preview file belongs to user projects.
        """
        if permissions.has_vendor_permissions():
            user_service.check_working_on_task(preview_file["task_id"])
        else:
            task = tasks_service.get_task(preview_file["task_id"])
            user_service.check_project_access(task["project_id"])
        return True

    def check_update_permissions(self, preview_file, data):
        task = tasks_service.get_task(preview_file["task_id"])
        user_service.check_project_access(task["project_id"])
        if not permissions.has_manager_permissions():
            user_service.check_working_on_task(task["entity_id"])
        return True



from flask import abort
from flask_jwt_extended import jwt_required
from flask_restful import current_app
from sqlalchemy.exc import StatementError

from studiopulsee.app.models.person import Person
from studiopulsee.app.services import deletion_service, index_service, persons_service
from studiopulsee.app.utils import permissions

from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource

from studiopulsee.app.mixin import ArgsMixin

from studiopulsee.app.services.exception import (
    DepartmentNotFoundException,
    WrongParameterException,
    PersonInProtectedAccounts,
)
from studiopulsee.app.models.department import Department

from studiopulsee.app import config


class PersonsResource(BaseModelsResource, ArgsMixin):
    def __init__(self):
        BaseModelsResource.__init__(self, Person)

    def all_entries(self, query=None, relations=False):
        if query is None:
            query = self.model.query

        if permissions.has_manager_permissions():
            if self.get_bool_parameter("with_pass_hash"):
                return [
                    person.serialize(relations=relations)
                    for person in query.all()
                ]
            else:
                return [
                    person.serialize_safe(relations=relations)
                    for person in query.all()
                ]
        else:
            return [
                person.present_minimal(relations=relations)
                for person in query.all()
            ]

    def post(self):
        abort(405)

    def check_read_permissions(self):
        return True


class PersonResource(BaseModelResource, ArgsMixin):
    def __init__(self):
        BaseModelResource.__init__(self, Person)
        self.protected_fields += ["password"]

    def check_read_permissions(self, instance):
        return True

    def check_update_permissions(self, instance_dict, data):
        if instance_dict["id"] != persons_service.get_current_user()["id"]:
            self.check_escalation_permissions(instance_dict, data)
        else:
            data.pop("role", None)
        return instance_dict

    def check_delete_permissions(self, instance_dict):
        if instance_dict["id"] == persons_service.get_current_user()["id"]:
            raise permissions.PermissionDenied
        self.check_escalation_permissions(instance_dict)
        return instance_dict

    def check_escalation_permissions(self, instance_dict, data=None):
        if permissions.admin_permission.can():
            return True
        else:
            raise permissions.PermissionDenied

    @jwt_required()
    def get(self, instance_id):
        """
        Retrieves the given person.
        """
        relations = self.get_bool_parameter("relations")

        try:
            instance = self.get_model_or_404(instance_id)
            result = self.serialize_instance(instance, relations=relations)
            self.check_read_permissions(result)
            result = self.clean_get_result(result)

        except StatementError as exception:
            current_app.logger.error(str(exception), exc_info=1)
            return {"message": str(exception)}, 400

        except ValueError:
            abort(404)

        return result, 200

    def serialize_instance(self, instance, relations=False):
        if permissions.has_manager_permissions():
            return instance.serialize_safe(relations=relations)
        else:
            return instance.present_minimal(relations=relations)

    def pre_update(self, instance_dict, data):
        if (
            data.get("active", False)
            and not instance_dict.get("active", False)
            and persons_service.is_user_limit_reached()
        ):
            raise WrongParameterException("User limit reached.")
        if (
            data.get("active") is False
            and instance_dict["email"] in config.PROTECTED_ACCOUNTS
        ):
            raise PersonInProtectedAccounts(
                "Can't set this person as inactive it's a protected account."
            )
        return data

    def post_update(self, instance_dict):
        persons_service.clear_person_cache()
        index_service.remove_person_index(instance_dict["id"])
        person = persons_service.get_person_raw(instance_dict["id"])
        if person.active:
            index_service.index_person(person)
        instance_dict["departments"] = [
            str(department.id) for department in self.instance.departments
        ]
        return instance_dict

    def post_delete(self, instance_dict):
        persons_service.clear_person_cache()
        return instance_dict

    def update_data(self, data, instance_id):
        if "password" in data:
            del data["password"]
        if "departments" in data:
            try:
                departments = []
                for department_id in data["departments"]:
                    department = Department.get(department_id)
                    if department is not None:
                        departments.append(department)
            except StatementError:
                raise DepartmentNotFoundException()
            data["departments"] = departments
        return data

    @jwt_required()
    def delete(self, instance_id):
        """
        Delete a person corresponding at given ID and return it as a JSON
        object.
        """
        force = self.get_force()
        person = self.get_model_or_404(instance_id)
        person_dict = person.serialize()
        self.check_delete_permissions(person_dict)
        self.pre_delete(person_dict)
        deletion_service.remove_person(instance_id, force=force)
        index_service.remove_person_index(instance_id)
        self.emit_delete_event(person_dict)
        self.post_delete(person_dict)
        return "", 204



from flask import abort
from studiopulsee.app.models.time_spent import TimeSpent
from sqlalchemy import func

from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource


class TimeSpentsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, TimeSpent)

    def apply_filters(self, query, options):
        query = super(TimeSpentsResource, self).apply_filters(query, options)
        start_date = options.get("start_date", None)
        end_date = options.get("end_date", None)
        if not start_date and not end_date:
            return query

        if None in [start_date, end_date]:
            abort(
                400,
                "If querying for a range of dates, both a `start_date` and an "
                "`end_date` must be given.",
            )

        return query.filter(
            self.model.date.between(
                func.cast(start_date, TimeSpent.date.type)
            ),
            func.cast(end_date, TimeSpent.date.type),
        )


class TimeSpentResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, TimeSpent)



from studiopulsee.app.models.day_off import DayOff
from studiopulsee.app.models.time_spent import TimeSpent

from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource

from studiopulsee.app.services import user_service
from studiopulsee.app.utils import date_helpers


class DayOffsResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, DayOff)

    def check_create_permissions(self, data):
        return user_service.check_day_off_access(data)

    def update_data(self, data):
        data["date"] = date_helpers.get_date_from_string(data["date"])
        return data

    def post_creation(self, instance):
        time_spents = TimeSpent.delete_all_by(
            person_id=instance.person_id, date=instance.date
        )
        return instance.serialize()


class DayOffResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, DayOff)

    def check_delete_permissions(self, instance):
        return user_service.check_day_off_access(instance)

    def check_read_permissions(self, instance):
        return user_service.check_day_off_access(instance)



import datetime

from sqlalchemy_utils import UUIDType
from studiopulsee.app.utils import fields
import sqlalchemy as sa


class BaseMixin(object):
    id = sa.Column(
        UUIDType(binary=False), primary_key=True, default=fields.gen_uuid
    )

    # Audit fields
    created_at = sa.Column(sa.DateTime, default=datetime.datetime.utcnow)
    updated_at = sa.Column(
        sa.DateTime,
        default=datetime.datetime.utcnow,
        onupdate=datetime.datetime.utcnow,
    )



from studiopulsee.app.models.attachment_file import AttachmentFile

from studiopulsee.app.blueprints.crud.base import BaseModelResource, BaseModelsResource

from studiopulsee.app.services import tasks_service, user_service


class AttachmentFilesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, AttachmentFile)


class AttachmentFileResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, AttachmentFile)

    def check_read_permissions(self, instance):
        attachment_file = instance
        comment = tasks_service.get_comment(attachment_file["comment_id"])
        task = tasks_service.get_task(comment["object_id"])
        user_service.check_project_access(task["project_id"])
        user_service.check_entity_access(task["entity_id"])
        return True



from studiopulsee.app.models.output_type import OutputType
from studiopulsee.app.blueprints.crud.base import BaseModelResource, BaseModelsResource


class OutputTypesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, OutputType)

    def check_read_permissions(self):
        return True


class OutputTypeResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, OutputType)

    def check_read_permissions(self, instance):
        return True



import traceback

from flask import Flask, jsonify, current_app
from flasgger import Swagger
from flask_jwt_extended import JWTManager
from flask_principal import Principal, identity_changed, Identity
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_mail import Mail

from jwt import ExpiredSignatureError
from babel.core import UnknownLocaleError
from meilisearch.errors import (
    MeilisearchApiError,
    MeilisearchCommunicationError,
)

from studiopulsee.app import config, swagger
from studiopulsee.app.stores import auth_tokens_store, file_store
from studiopulsee.app.services.exception import (
    ModelWithRelationsDeletionException,
    PersonNotFoundException,
    WrongIdFormatException,
    WrongParameterException,
    WrongTaskTypeForEntityException,
)

from studiopulsee.app.utils import cache, fs, logs, monitoring
from studiopulsee.app.utils.flask import ParsedUserAgent, ORJSONProvider

app = Flask(__name__)
app.json = ORJSONProvider(app)
app.request_class.user_agent_class = ParsedUserAgent
app.config.from_object(config)

monitoring.init_monitoring(app)

if config.LOGS_MODE == "ovh":
    logs.configure_logs_ovh(app)

db = SQLAlchemy(app)
migrate = Migrate(app, db)  # DB schema migration features
app.extensions["sqlalchemy"].db = db

app.secret_key = app.config["SECRET_KEY"]
jwt = JWTManager(app)  # JWT auth tokens
Principal(app)  # Permissions
cache.cache.init_app(app)  # Function caching
mail = Mail()
mail.init_app(app)  # To send emails
swagger = Swagger(
    app, template=swagger.swagger_template, config=swagger.swagger_config
)


@app.teardown_appcontext
def shutdown_session(exception=None):
    db.session.remove()


@app.errorhandler(404)
def page_not_found(error):
    return jsonify(error=True, message=str(error)), 404


@app.errorhandler(WrongIdFormatException)
def id_parameter_format_error(error):
    return (
        jsonify(
            error=True,
            message="One of the ID sent in parameter is not properly formatted.",
        ),
        400,
    )


@app.errorhandler(WrongParameterException)
def wrong_parameter(error):
    return jsonify(error=True, message=str(error)), 400


@app.errorhandler(ExpiredSignatureError)
def wrong_token_signature(error):
    return jsonify(error=True, message=str(error)), 401


@app.errorhandler(ModelWithRelationsDeletionException)
def try_delete_model_with_relations(error):
    return jsonify(error=True, message=str(error)), 400


@app.errorhandler(WrongTaskTypeForEntityException)
def wrong_task_type_for_entity(error):
    return jsonify(error=True, message=str(error)), 400


@app.errorhandler(UnknownLocaleError)
def wrong_locale_label(error):
    return jsonify(error=True, message=str(error)), 400


@app.errorhandler(MeilisearchCommunicationError)
def indexer_not_reachable(error):
    current_app.logger.error("Indexer not reachable")
    return jsonify(error=True, message="Indexer not reachable"), 500


@app.errorhandler(MeilisearchApiError)
def indexer_key_error(error):
    if error.code == "invalid_api_key":
        current_app.logger.error("The indexer key is rejected")
        return jsonify(error=True, message="The indexer key is rejected"), 500
    else:
        raise error


if not config.DEBUG:

    @app.errorhandler(Exception)
    def server_error(error):
        stacktrace = traceback.format_exc()
        current_app.logger.error(stacktrace)
        return (
            jsonify(error=True, message=str(error), stacktrace=stacktrace),
            500,
        )


def configure_auth():
    from studiopulsee.app.services import persons_service

    @jwt.token_in_blocklist_loader
    def check_if_token_is_revoked(_, payload):
        return auth_tokens_store.is_revoked(payload)

    @jwt.user_lookup_loader
    def add_permissions(_, payload):
        try:
            user = persons_service.get_person(payload["user_id"])
            if user is not None:
                identity_changed.send(
                    current_app._get_current_object(),
                    identity=Identity(user["id"]),
                )
            return user
        except PersonNotFoundException:
            return None


def load_api(app):
    from studiopulsee.app import api

    api.configure(app)

    fs.mkdir_p(app.config["TMP_DIR"])
    configure_auth()


file_store.configure_storages(app)
load_api(app)



from studiopulsee.app.models.output_file import OutputFile
from studiopulsee.app.models.entity import Entity
from studiopulsee.app.models.project import Project
from studiopulsee.app.services import user_service, entities_service
from studiopulsee.app.utils import permissions

from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource


class OutputFilesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, OutputFile)

    def check_read_permissions(self):
        user_service.block_access_to_vendor()
        return True

    def add_project_permission_filter(self, query):
        if permissions.has_admin_permissions():
            return query
        else:
            query = (
                query.join(Entity, OutputFile.entity_id == Entity.id)
                .join(Project)
                .filter(user_service.build_related_projects_filter())
            )
            return query


class OutputFileResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, OutputFile)

    def check_read_permissions(self, instance):
        entity = entities_service.get_entity(instance["entity_id"])
        user_service.check_project_access(entity["project_id"])
        user_service.check_entity_access(entity["id"])
        return True

    def check_update_permissions(self, output_file, data):
        if permissions.has_manager_permissions():
            return True
        else:
            return user_service.check_working_on_entity(
                output_file["temporal_entity_id"] or output_file["entity_id"]
            )



import sys
import redis

from studiopulsee.app import config


try:
    revoked_tokens_store = redis.StrictRedis(
        host=config.KEY_VALUE_STORE["host"],
        port=config.KEY_VALUE_STORE["port"],
        db=config.AUTH_TOKEN_BLACKLIST_KV_INDEX,
        decode_responses=True,
    )
    revoked_tokens_store.get("test")
except redis.ConnectionError:
    try:
        import fakeredis

        revoked_tokens_store = fakeredis.FakeStrictRedis()
    except BaseException:
        print("Cannot access to the required Redis instance")
        sys.exit(1)


def add(key, token, ttl=None):
    """
    Store a token with key as access key.
    """
    return revoked_tokens_store.set(key.encode("utf-8"), token, ex=ttl)


def get(key):
    """
    Retrieve auth token corresponding at given key.
    """
    value = revoked_tokens_store.get(key)
    if value is not None and hasattr(value, "decode"):
        value = value.decode("utf-8")
    return value


def delete(key):
    """
    Remove auth token corresponding at given key.
    """
    return revoked_tokens_store.delete(key.encode("utf-8"))


def keys():
    """
    Get all keys available in the store.
    """
    keys = revoked_tokens_store.keys()
    if len(keys) > 0 and hasattr(keys[0], "decode"):
        return [x.decode("utf-8") for x in revoked_tokens_store.keys()]
    else:
        return [x for x in revoked_tokens_store.keys()]


def clear():
    """
    Clear all auth token stored in the store.
    """
    for key in keys():
        delete(key)


def is_revoked(decrypted_token):
    """
    Tell if a stored auth token is revoked or not.
    """
    jti = decrypted_token["jti"]
    is_revoked = get(jti)
    return (is_revoked is None) or (is_revoked == "true")



from studiopulsee.app.blueprints.export.csv.base import BaseCsvExport

from studiopulsee.app.models.person import Person


class PersonsCsvExport(BaseCsvExport):
    def __init__(self):
        BaseCsvExport.__init__(self)
        self.file_name = "people_export"

    def build_headers(self):
        return ["Last Name", "First Name", "Email", "Phone", "Role", "Active"]

    def build_query(self):
        return Person.query.order_by(Person.last_name, Person.first_name)

    def build_row(self, person):
        active = "yes"
        if not person.active:
            active = "no"
        return [
            person.last_name,
            person.first_name,
            person.email,
            person.phone,
            person.role,
            active,
        ]



from studiopulsee.app.blueprints.crud.base import BaseModelsResource, BaseModelResource

from studiopulsee.app.models.entity import Entity
from studiopulsee.app.models.project import Project
from studiopulsee.app.models.working_file import WorkingFile
from studiopulsee.app.services import user_service, tasks_service, files_service
from studiopulsee.app.utils import permissions


class WorkingFilesResource(BaseModelsResource):
    def __init__(self):
        BaseModelsResource.__init__(self, WorkingFile)

    def check_read_permissions(self):
        """
        Overriding so that people without admin credentials can still access
        this resource.
        """
        user_service.block_access_to_vendor()
        return True

    def add_project_permission_filter(self, query):
        """
        Filtering to keep only the files from projects available to the user's
        team. Allows projects that are no longer open.
        """
        if permissions.has_admin_permissions():
            return query
        else:
            query = (
                query.join(Entity, WorkingFile.entity_id == Entity.id)
                .join(Project)
                .filter(user_service.build_team_filter())
            )
            return query


class WorkingFileResource(BaseModelResource):
    def __init__(self):
        BaseModelResource.__init__(self, WorkingFile)

    def check_read_permissions(self, instance):
        working_file = files_service.get_working_file(instance["id"])
        task = tasks_service.get_task(working_file["task_id"])
        user_service.check_project_access(task["project_id"])
        user_service.check_entity_access(task["entity_id"])
        return True

    def check_update_permissions(self, instance, data):
        working_file = files_service.get_working_file(instance["id"])
        task = tasks_service.get_task(working_file["task_id"])
        user_service.check_project_access(task["project_id"])
        user_service.check_entity_access(task["entity_id"])
        return True



from studiopulsee.app.blueprints.source.csv.base import BaseCsvImportResource

from studiopulsee.app.models.person import Person
from studiopulsee.app.services import index_service
from studiopulsee.app.utils import permissions


class PersonsCsvImportResource(BaseCsvImportResource):
    def post(self, **kwargs):
        """
        Import persons via a .csv file.
        ---
        tags:
            - Import
        consumes:
          - multipart/form-data
        parameters:
          - in: formData
            name: file
            type: file
            required: true
        responses:
            201:
                description: The lists of imported persons.
            400:
                description: The .csv file is not properly formatted.
        """
        return super().post(**kwargs)

    def check_permissions(self):
        return permissions.check_admin_permissions()

    def import_row(self, row):
        first_name = row["First Name"]
        last_name = row["Last Name"]
        email = row["Email"]
        phone = row.get("Phone", None)
        role = row.get("Role", None)

        role_map = {
            "Studio Manager": "admin",
            "Production Manager": "manager",
            "Supervisor": "supervisor",
            "Artist": "user",
            "Client": "client",
            "Vendor": "vendor",
        }

        data = {"first_name": first_name, "last_name": last_name}
        if role is not None and role != "":
            if role in role_map.keys():
                role = role_map[role]
            data["role"] = role
        if phone is not None and phone != "":
            data["phone"] = phone

        person = Person.get_by(email=email)
        if person is None:
            data["email"] = email
            data["password"] = None
            person = Person.create(**data)
        elif self.is_update:
            person.update(data)
        index_service.index_person(person)
        return person.serialize_safe()



from setuptools import find_packages, setup

setup(
    name='studiopulsee',
    version='0.17.1',  # Set your version here
    description='It\'s a Python library for VFX project-based LAN deployment',
    author='Hitech Enterprise',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
                'aiohttp==3.9.5',
                'aiosignal==1.3.1',
                'alembic==1.13.2',
                'aniso8601==9.0.1',
                'annotated-types==0.7.0',
                'async-timeout==4.0.3',
                'attrs==23.2.0',
                'Babel==2.13.1',
                'bcrypt==4.1.3',
                'bidict==0.23.1',
                'blinker==1.8.2',
                'boto3==1.34.135',
                'botocore==1.34.135',
                'cachelib==0.9.0',
                'camel-converter==3.1.2',
                'certifi==2024.6.2',
                'cffi==1.16.0',
                'charset-normalizer==3.3.2',
                'click==8.1.7',
                'cryptography==42.0.8',
                'debtcollector==3.0.0',
                'discord.py==2.3.2',
                'dnspython==2.6.1',
                'email-validator==2.1.0.post1',
                'ffmpeg-python==0.2.0',
                'fido2==1.1.2',
                'flasgger==0.9.7.1',
                'Flask==3.0.0',
                'Flask-Bcrypt==1.0.1',
                'Flask-Caching==2.1.0',
                'Flask-Fixtures==0.3.8',
                'flask-fs2==0.7.23',
                'Flask-JWT-Extended==4.5.3',
                'Flask-Mail==0.9.1',
                'Flask-Migrate==4.0.5',
                'Flask-Principal==0.4.0',
                'Flask-RESTful==0.3.10',
                'Flask-SocketIO==5.3.6',
                'Flask-SQLAlchemy==3.1.1',
                'frozenlist==1.4.1',
                'future==1.0.0',
                'gazu==0.9.14',
                'gevent==23.9.1',
                'gevent-websocket==0.10.1',
                'greenlet==3.0.3',
                'gunicorn==21.2.0',
                'h11==0.14.0',
                'idna==3.7',
                'iso8601==2.1.0',
                'isoweek==1.3.3',
                'itsdangerous==2.1.2',
                'Jinja2==3.1.2',
                'jmespath==1.0.1',
                'jsonschema==4.22.0',
                'jsonschema-specifications==2023.12.1',
                'keystoneauth1==5.6.0',
                'ldap3==2.9.1',
                'Mako==1.3.5',
                'MarkupSafe==2.1.5',
                'matterhook==0.2',
                'meilisearch==0.28.4',
                'mistune==3.0.2',
                'msgpack==1.0.8',
                'multidict==6.0.5',
                'netaddr==1.3.0',
                'netifaces==0.11.0',
                'numpy==1.24.4',
                'opencv-python==4.8.1.78',
                'OpenTimelineIO==0.15.0',
                'orjson==3.9.10',
                'os-service-types==1.7.0',
                'oslo.config==9.4.0',
                'oslo.i18n==6.3.0',
                'oslo.serialization==5.4.0',
                'oslo.utils==7.1.0',
                'packaging==24.1',
                'pbr==6.0.0',
                'Pillow==10.1.0',
                'psutil==5.9.6',
                'psycopg==3.1.12',
                'psycopg-binary==3.1.12',
                'pyaaf2==1.6.0',
                'pyasn1==0.6.0',
                'pycparser==2.22',
                'pydantic==2.7.4',
                'pydantic_core==2.18.4',
                'PyJWT==2.8.0',
                'pyotp==2.9.0',
                'pyparsing==3.1.2',
                'python-dateutil==2.8.2',
                'python-engineio==4.9.1',
                'python-keystoneclient==5.4.0',
                'python-nomad==2.0.0',
                'python-slugify==8.0.1',
                'python-socketio==5.10.0',
                'python-swiftclient==4.6.0',
                'pytz==2023.3.post1',
                'PyYAML==6.0.1',
                'redis==5.0.1',
                'referencing==0.35.1',
                'requests==2.31.0',
                'rfc3986==2.0.0',
                'rpds-py==0.18.1',
                'rq==1.15.1',
                's3transfer==0.10.2',
                'simple-websocket==1.0.0',
                'six==1.16.0',
                'slackclient==2.9.4',
                'SQLAlchemy==2.0.23',
                'SQLAlchemy-Utils==0.41.1',
                'stevedore==5.2.0',
                'text-unidecode==1.3',
                'typing_extensions==4.12.2',
                'tzdata==2024.1',
                'ua-parser==0.18.0',
                'urllib3==2.2.2',
                'websocket-client==1.8.0',
                'Werkzeug==3.0.1',
    ],
    
    entry_points={
        'console_scripts': [
            'studiopulsee=studiopulsee.app:main',  # Adjust this to your main function
        ],
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Multimedia :: Graphics',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from ua_parser import user_agent_parser
from werkzeug.user_agent import UserAgent
from werkzeug.utils import cached_property
from flask.json.provider import JSONProvider
from flask import make_response
import orjson

orjson_options = orjson.OPT_NON_STR_KEYS


def output_json(data, code, headers=None):
    """Makes a Flask response with a JSON encoded body"""
    dumped = orjson.dumps(data, option=orjson_options)

    resp = make_response(dumped, code)
    resp.headers.extend(headers or {})
    return resp


class ORJSONProvider(JSONProvider):
    def __init__(self, *args, **kwargs):
        self.options = kwargs
        super().__init__(*args, **kwargs)

    def loads(self, s, **kwargs):
        return orjson.loads(s)

    def dumps(self, obj, **kwargs):
        return orjson.dumps(obj, option=orjson_options).decode("utf-8")


class ParsedUserAgent(UserAgent):
    @cached_property
    def _details(self):
        return user_agent_parser.Parse(self.string)

    @property
    def platform(self):
        return self._details["os"]["family"]

    @property
    def browser(self):
        return self._details["user_agent"]["family"]

    @property
    def version(self):
        return ".".join(
            part
            for key in ("major", "minor", "patch")
            if (part := self._details["user_agent"][key]) is not None
        )



from flask import Blueprint
from studiopulsee.app.utils.api import configure_api_from_blueprint

from studiopulsee.app.blueprints.entities.resources import (
    EntityPreviewFilesResource,
    EntityNewsResource,
    EntityTimeSpentsResource,
)


routes = [
    ("/data/entities/<entity_id>/news", EntityNewsResource),
    ("/data/entities/<entity_id>/preview-files", EntityPreviewFilesResource),
    ("/data/entities/<entity_id>/time-spents", EntityTimeSpentsResource),
]

blueprint = Blueprint("entities", "entities")
api = configure_api_from_blueprint(blueprint, routes)
